

bool DEPTH_BIAS = false;
bool BORDER_CLAMP = false;

float3 lightPos;
float specularPower = 64.0;

float depth = 0.1;

texture tex;
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
sampler2D color_map = sampler_state
{
	Texture = <tex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler2D displacement_map = sampler_state
{
	Texture = <tex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = None;
};

sampler2D cone_map = sampler_state
{
	Texture = <tex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler3D cone_map3D = sampler_state
{
	Texture = <tex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler3D sphere_map = sampler_state
{
	Texture = <tex>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

// Matrix
// ---------------------------------------------
float4x4 matW 	: World;
float4x4 matV  : View;
float4x4 matVI 	: ViewInverse;
float4x4 matP  : Projection;
float4x4 matWVP : WorldViewProjection;


//float4x4 matWIT : WorldInverseTranspose;
float4 lAmb  : COLOR <String uiname="Ambient Color";>  = {0.15, 0.15, 0.15, 1};
float4 lDiff : COLOR <String uiname="Diffuse Color";>  = {0.85, 0.85, 0.85, 1};
float4 lSpec : COLOR <String uiname="Specular Color";> = {0.35, 0.35, 0.35, 1};

#include "BUMPS.fxh"

// Structs
// ---------------------------------------------
struct vertexInput
{
    float4 pos: POSITION;
    float3 norm: NORMAL;
    float2 uv0: TEXCOORD0;
    float3 tang: TANGENT;
    float3 bino: BINORMAL;
};

struct pixelInput
{
	float4 hpos: POSITION;
	float2 uv0: TEXCOORD0;
	float3 eyeVec: TEXCOORD1;
	float3 lightVec: TEXCOORD2;
	float3 normalVec: TEXCOORD3;
};

pixelInput simple_vs(vertexInput IN) 
{
    pixelInput OUT;

    // Processing
    OUT.hpos = mul(IN.pos, matWVP);
    OUT.uv0 = mul(IN.uv0,tTex);
    
   	// World vertex
	float3 vertexPos = mul(IN.pos, matW);
	
	// norm vector (Only used by POM)
	OUT.normalVec = mul(IN.norm, matW);
	
	// View vector
	OUT.eyeVec = matVI[3].xyz - vertexPos;
	
	// Light vector
	OUT.lightVec = lightPos - vertexPos;
	
    return OUT;
}

pixelInput bump_vs(vertexInput IN) 
{
    pixelInput OUT;

    // Vertex in clip space    
    OUT.hpos = mul(IN.pos, matWVP);
    
    // Copy texcoord
    OUT.uv0 = mul(IN.uv0 ,tTex);
	// World vertex
	float3 vertexPos = mul(IN.pos, matW);
	// Normal vector (Used by Texture Mapping and POM)
	OUT.normalVec = mul(IN.norm, matW);
    // Tangent space (Tangent, binormal, normal)
    float3x3 tangentMap = float3x3(IN.tang, IN.bino, IN.norm);
    tangentMap = mul(tangentMap, matW);
	// View vector
	float3 eyeVec = vertexPos - matVI[3].xyz;	
	OUT.eyeVec = mul(tangentMap, eyeVec);
	
	// Light vector
	float3 lightVec = lightPos - vertexPos;
	OUT.lightVec = mul(tangentMap, lightVec);
	
    return OUT;
}

pixelInput displacement_vs(vertexInput IN) 
{
    pixelInput OUT;

    // Copy texcoord
    OUT.uv0 = mul(IN.uv0,tTex);

    // Read height
    float4 atex = tex2Dlod(displacement_map, float4(OUT.uv0, 0.0f, 0.0f));
     //use luminance for now
    float height;
    height = 0.27f * atex.r + 0.67f * atex.g + 0.06f * atex.b;

    // Vertex in clip space
    float scaleDepth = 	100.0f;
    IN.pos = float4(IN.pos - IN.norm * height * depth * scaleDepth, 1.0f);
    OUT.hpos = mul(IN.pos, matWVP);
    
    // World vertex
	float3 vertexPos = mul(IN.pos, matW);
	
	// Normal
	float3 norm;
	norm.xy = atex.xy * 2.0f - 1.0f;
	norm.y = -norm.y;
	norm.z = sqrt(1.0 - dot(atex.xy, atex.xy));
    
    float3x3 tangentMap = float3x3(IN.tang, IN.bino, IN.norm);	
	
	// Transform the normal by the tangent space instead of putting
	// the eye vector and light vector in the tangent space
	norm = mul(norm, tangentMap);
	OUT.normalVec = mul(norm, matW);
	
	// View vector
	OUT.eyeVec = vertexPos - matVI[3].xyz;	
	
	// Light vector
	OUT.lightVec = lightPos - vertexPos;

	return OUT;
}



float4 phongShading(float3 norm, float3 lightVec, float3 eyeVec, float3 materialColor)
{
	float3 halfVec = normalize(lightVec + eyeVec);
        //shades.y*atten
            //compute blinn lighting
        float4 shades = lit(dot(norm, lightVec), dot(norm, halfVec), specularPower);
	float diffuseInt = shades.y;
	//
	//float specularInt = saturate(dot(halfVec, norm));
	//specularInt = pow(specularInt, specularPower);
	
	    //reflection vector (view space)
    float3 R = normalize(2 * dot(norm, lightVec) * norm - lightVec);

    //normalized view direction (view space)
    float3 V = normalize(eyeVec);

    //calculate specular light
    float specularInt = pow(max(dot(R, V),0), specularPower*.2);
    
	float4 finalColor;
	finalColor.xyz = (lAmb + lDiff*diffuseInt) + specularInt*lSpec;
	finalColor.xyz *= materialColor;
	finalColor.w = 1.0f;
	return finalColor;
}

float4 displacement_ps(pixelInput IN) : COLOR
{
	// View and light vectors
	float3 v = normalize(IN.eyeVec);
	float3 l1 = normalize(IN.lightVec);
	float3 n = normalize(IN.normalVec);
	
	// Diffuse texture color
	float3 col = tex2D(color_map, IN.uv0).xyz;
	
	return phongShading(n, l1, -v, col);
}

float4 bump_ps(float2 uv0, float3 eyeVec, float3 lightVec) : COLOR
{
	if (BORDER_CLAMP)
	{
		if (uv0.x < 0.0) discard;
		if (uv0.x > tTex[0].x) discard;
		if (uv0.y < 0.0) discard;
		if (uv0.y > tTex[1].y) discard;
	}

	// View and light vectors
	float3 v = normalize(eyeVec);
	float3 l1 = normalize(lightVec);
	
	// Diffuse texture color
	float3 col = tex2D(color_map, uv0).xyz;

	// Bump map
	float3 n = tex2D(cone_map, uv0);
	n.xy = n.xy * 2.0 - 1.0;
	n.y = -n.y;
	n.z = sqrt(1.0 - dot(n.xy, n.xy));
	
	return phongShading(n, l1, -v, col);
}

float4 texture_ps(pixelInput IN) : COLOR
{
	// View and light vectors
	float3 n = normalize(IN.normalVec);
	float3 v = normalize(IN.eyeVec);
	float3 l1 = normalize(IN.lightVec);
	
	// Diffuse texture color
	float3 col = tex2D(color_map, IN.uv0).xyz;

	return phongShading(n, l1, v, col);
}

float4 sphere_ps(pixelInput IN) : COLOR
{
	// Texture width MUST be equal to texture height
	IN.eyeVec.xy *= depth * 15.0f/256.0f * size_source.x;
	IN.uv0  = cone3D_fixed(IN.uv0,IN.eyeVec, cone_map3D);
	return bump_ps(IN.uv0, IN.eyeVec, IN.lightVec);
}


float4 cone3D_fixed_ps(pixelInput IN) : COLOR
{
	IN.uv0  = cone3D_fixed(IN.uv0,IN.eyeVec, cone_map3D);
	return bump_ps(IN.uv0, IN.eyeVec, IN.lightVec);
}

float4 cone_ps(pixelInput IN) : COLOR
{
       IN.uv0  = cone(IN.uv0,IN.eyeVec, cone_map);
       return bump_ps(IN.uv0, IN.eyeVec, IN.lightVec);
}

float4 pom_ps(pixelInput IN) : COLOR
{
	IN.uv0  = parallax_occulsion(IN.uv0,IN.eyeVec,IN.lightVec, IN.normalVec, cone_map);
	return bump_ps(IN.uv0, IN.eyeVec, IN.lightVec);
}

float4 relief_ps(pixelInput IN) : COLOR
{
	IN.uv0  = relief(IN.uv0,IN.eyeVec,IN.lightVec, cone_map);
	return bump_ps(IN.uv0, IN.eyeVec, IN.lightVec);
}

float4 parallax_ps(pixelInput IN) : COLOR
{
	IN.uv0  = parallax(IN.uv0,IN.eyeVec,IN.lightVec, cone_map);
	return bump_ps(IN.uv0, IN.eyeVec, IN.lightVec);
}

float4 normal_ps(pixelInput IN) : COLOR
{
	return bump_ps(IN.uv0, IN.eyeVec, IN.lightVec);
}

float4 mipmap_ps(pixelInput IN) : COLOR
{
	float2 absoluteTexcoord = IN.uv0 * float2(size_source.x, size_source.y);
	float2 dxAbsolute = ddx(absoluteTexcoord);
	float2 dyAbsolute = ddy(absoluteTexcoord);
	
	// Calculate mipmap
	// mipmap = max( sqrt((Du/Dx)^2 + (Dv/Dx)^2), sqrt((Du/Dy)^2 + (Dv/Dy)^2) )
	float2 dxdySquared = dxAbsolute * dxAbsolute + dyAbsolute * dyAbsolute;
	float texcoordDelta = max(dxdySquared.x, dxdySquared.y);
	float mipmapLevel = max(0.0, sqrt(texcoordDelta));
	
	// Change colors according with the mipmap
	float4 resultColor = float4(1.0f, 1.0f, 1.0f, 1.0f);
	if (mipmapLevel >= 0.0 && mipmapLevel < 1.0)
		resultColor = float4(1.0f, 0.0f, 0.0f, 1.0f);
	if (mipmapLevel >= 1.0 && mipmapLevel < 2.0)
		resultColor = float4(0.0f, 1.0f, 0.0f, 1.0f);
	if (mipmapLevel >= 2.0 && mipmapLevel < 3.0)
		resultColor = float4(0.0f, 0.0f, 1.0f, 1.0f);
	if (mipmapLevel >= 3.0 && mipmapLevel < 4.0)
		resultColor = float4(1.0f, 1.0f, 0.0f, 1.0f);
	if (mipmapLevel >= 4.0 && mipmapLevel < 5.0)
		resultColor = float4(0.0f, 1.0f, 1.0f, 1.0f);
	if (mipmapLevel >= 5.0 && mipmapLevel < 6.0)
		resultColor = float4(1.0f, 0.0f, 1.0f, 1.0f);
	if (mipmapLevel >= 6.0 && mipmapLevel < 7.0)
		resultColor = float4(0.5f, 0.5f, 0.5f, 1.0f);
	if (mipmapLevel > 7.0 && mipmapLevel <= 8.0)
		resultColor = float4(0.0f, 0.0f, 0.0f, 1.0f);
		
	return resultColor;
}

technique sphere_mapping
{
    pass P0
    {
		VertexShader = compile vs_3_0 bump_vs();
		PixelShader  = compile ps_3_0 sphere_ps();
    }
}

technique Tcone3D_fixed
{
    pass P0
    {
		VertexShader = compile vs_3_0 bump_vs();
		PixelShader  = compile ps_3_0 cone3D_fixed_ps();
    }
}

technique cone_mapping
{
    pass P0
    {
		VertexShader = compile vs_3_0 bump_vs();
		PixelShader  = compile ps_3_0 cone_ps();
    }
}

technique parallax_occlusion_mapping
{
    pass P0
    {          
        VertexShader = compile vs_3_0 bump_vs();
        PixelShader  = compile ps_3_0 pom_ps(); 
    }
}

technique relief_mapping
{
    pass P0
    {
		VertexShader = compile vs_3_0 bump_vs();
		PixelShader  = compile ps_3_0 relief_ps();
    }
}
technique normal_mapping
{
    pass P0
    {
         VertexShader = compile vs_3_0 bump_vs();
	 PixelShader  = compile ps_3_0 normal_ps();
    }
}

technique displacement_mapping
{
    pass P0
    {
        VertexShader = compile vs_3_0 displacement_vs();
        PixelShader  = compile ps_3_0 displacement_ps();
    }
}
technique parallax_mapping
{
    pass P0
    {
         VertexShader = compile vs_3_0 bump_vs();
         PixelShader  = compile ps_3_0 parallax_ps();
    }
}

technique mip_mapping
{
    pass P0
    {          
        VertexShader = compile vs_3_0 bump_vs();
        PixelShader  = compile ps_3_0 mipmap_ps();
    }
}

technique texture_mapping
{
    pass P0
    {          
        VertexShader = compile vs_3_0 simple_vs();
        PixelShader  = compile ps_3_0 texture_ps(); 
    }
}
